//
//  AsynchronousRequest.m
//  LoadContentDemo
//
//  Created by Brian Halpin on 21/11/2012.
//  Copyright (c) 2012 Brian Halpin. All rights reserved.
//

#import "AsynchronousRequest.h"

@interface AsynchronousRequest ()

- (void)callbackCompletion:(NSArray *)dataArray;

@end

@implementation AsynchronousRequest

+ (NSURLRequest *)createRequestWithURL:(NSString *)requestURL timeoutInterval:(double)timeout
{
    NSURLRequest *request = [NSURLRequest requestWithURL:[NSURL URLWithString:requestURL]
                                             cachePolicy:NSURLRequestUseProtocolCachePolicy
                                         timeoutInterval:timeout];
    
    return request;
}


#pragma mark - GCD asynchronous requests

- (void)makeAsynchronousRequest:(NSURLRequest *)request
{
    NSArray *requestsArray = [NSArray arrayWithObjects:request, nil];
    NSMutableArray *returnDataArray = [[NSMutableArray alloc] init];
    
    dispatch_queue_t callerQueue;
    
    callerQueue = dispatch_queue_create("com.urlrequest.queue", DISPATCH_QUEUE_SERIAL);
    
    dispatch_queue_t downloadQueue = dispatch_queue_create("com.download.queue", NULL);
    
    dispatch_async(downloadQueue, ^{
        for (NSURLRequest *request in requestsArray)
        {
            [returnDataArray addObject:[NSURLConnection sendSynchronousRequest:request returningResponse:nil error:nil]];
        }
        dispatch_async(callerQueue, ^{
            [self callbackCompletion:returnDataArray];
        });
    });
}

#pragma mark - Callback functions

- (void)callbackCompletion:(NSArray *)dataArray
{
    NSLog(@"Returned data: %@", dataArray);
    
    NSError* err = nil;
    
    for (NSData *jsonData in dataArray)
    {
    
        id jsonObject = [NSJSONSerialization JSONObjectWithData:jsonData
                                                     options:kNilOptions
                                                       error:&err];
        
        BOOL isValidJSON = [NSJSONSerialization isValidJSONObject:jsonObject];
        
        if (isValidJSON)
        {
            NSLog(@"jsonArray: %@", jsonObject);
        }
    }
    
}


@end
