//
//  CoreData.m
//  LoadContentDemo
//
//  Created by Brian Halpin on 23/11/2012.
//  Copyright (c) 2012 Brian Halpin. All rights reserved.
//

#import "CoreData.h"
#import "AppDelegate.h"

@interface CoreData ()

@property (nonatomic, strong) NSManagedObjectContext *managedObjectContext;

@end

@implementation CoreData


#pragma mark - Setters / Getters

- (NSManagedObjectContext *)managedObjectContext
{
    if (!_managedObjectContext)
    {
        AppDelegate *appDelegate = [[UIApplication sharedApplication] delegate];
        _managedObjectContext = appDelegate.managedObjectContext;
    }
    
    return _managedObjectContext;
}

#pragma mark - Set predicate & Query DB

+ (NSPredicate *)predicateForField:(NSString *)field withValue:(NSString *)value
{
    NSPredicate *predicate = [NSPredicate predicateWithFormat:@"%@ CONTAINS[c] %@", field, value];
    
    return predicate;
}

- (NSArray *)fetchAllResultsFromCoreData
{
    return [self fetchResultsFromCoreDataUsingPredicate:nil];
}

- (NSArray *)fetchResultsFromCoreDataUsingPredicate:(NSPredicate *)predicate
{
    
    NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
    NSEntityDescription *entity = [NSEntityDescription
                                   entityForName:@"CORE_DATA_ENTITY_NAME" inManagedObjectContext:self.managedObjectContext];
    
    [fetchRequest setEntity:entity];
    
    // Check if predicate is set
    if (predicate)
    {
        [fetchRequest setPredicate:predicate];
    }
    
    NSError *error = nil;
    
    NSArray *fetchedManagedObjects = [self.managedObjectContext executeFetchRequest:fetchRequest error:&error];
    NSMutableArray *fetchedResults = [[NSMutableArray alloc] initWithCapacity:[fetchedManagedObjects count]];
    
    for (NSManagedObject *managedObject in fetchedManagedObjects)
    {
        // Convert managedObject to Dictionary
        NSArray *keys = [[[managedObject entity] attributesByName] allKeys];
        NSDictionary *managedObjectDictionary = [managedObject dictionaryWithValuesForKeys:keys];
        
        [fetchedResults addObject:managedObjectDictionary];
    }
    
    NSLog(@"fetchedResults: %@", fetchedResults);
    
    return [NSArray arrayWithArray:fetchedResults];
}

- (void)persistArrayOfDictionaries:(NSArray *)arrayOfDictionaries forEntityName:(NSString *)entityName
{
    
    [arrayOfDictionaries enumerateObjectsUsingBlock:^(id diciontaryObject, NSUInteger idx, BOOL *stop) {
    
        NSManagedObject *managedObject = [NSEntityDescription insertNewObjectForEntityForName:entityName
                                                                       inManagedObjectContext:self.managedObjectContext];
    
        [managedObject setValuesForKeysWithDictionary:diciontaryObject];
    
        NSError *error;
        
        if (![self.managedObjectContext save:&error]) {
            NSLog(@"Problem. Could not persist data: %@", [error localizedDescription]);
        }
    }];
    
}

@end
