//
//  ViewController.m
//  LoadContentDemo
//
//  Created by Brian Halpin on 21/11/2012.
//  Copyright (c) 2012 Brian Halpin. All rights reserved.
//

#import "ViewController.h"
#import "AsynchronousRequest.h"
#import "Reachability.h"
#import "Connection.h"

@interface ViewController ()

@property (nonatomic, strong) Reachability *reachability;

- (void)downloadContent;
- (void)addReachabilityNotification;

@end

@implementation ViewController

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
        
        [self addReachabilityNotification];
        
    }
    return self;
}

#pragma mark - Lazy property

- (Reachability *)reachability
{
    if (!_reachability)
    {
        _reachability = [Reachability reachabilityForInternetConnection];
    }
    
    return _reachability;
}

#pragma mark - View

- (void)viewDidLoad
{
    [super viewDidLoad];
	// Do any additional setup after loading the view.
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark - Connection requests

// The blocks will be called on a background thread
- (void)addReachabilityNotification
{    
    // allocate a reachability object
    Reachability* reach = [Reachability reachabilityWithHostname:@"www.google.com"];
    
    // set the blocks
    reach.reachableBlock = ^(Reachability*reach)
    {
        NSLog(@"Internet connection established - Reachable!");
        [self downloadContent];
    };
    
    reach.unreachableBlock = ^(Reachability*reach)
    {
        NSLog(@"Connection lost - Unreachable!");
    };
    
    // start the notifier which will cause the reachability object to retain itself!
    [reach startNotifier];
}

- (void)downloadContent
{
    BOOL connectionAvailable = [Connection networkConnectionAvailable];
    
    NSLog(connectionAvailable ? @"Connection available - begin download" : @"No connection available.");
    
    if (connectionAvailable)
    {
        NSURLRequest *urlRequest = [AsynchronousRequest createRequestWithURL:@"https://api.twitter.com/1.1/statuses/home_timeline.json" timeoutInterval:30.0];
        
        AsynchronousRequest *request = [[AsynchronousRequest alloc] init];
        [request makeAsynchronousRequest:urlRequest];
    }
}

@end
